#! /usr/local/bin/perl
# 
# RTG Target File Generator 
#
########################################################################
# $Id: rtgtargmkr.pl,v 1.3 2004/07/08 19:32:22 btoneill Exp $
########################################################################
# $Log: rtgtargmkr.pl,v $
# Revision 1.3  2004/07/08 19:32:22  btoneill
# Moved standard interface information to a module
# Added order file parsing for modules
# various other cleanups
#
# Revision 1.2  2004/07/08 16:42:52  btoneill
# Added changes to make Extreme switches work better and to
# remove interfaces that had no data for them
#
# Revision 1.1  2004/06/24 20:08:13  btoneill
# Initial revision
#
#
########################################################################

use vars qw($community $defbits $output $DEBUG $DBOFF $INFO @ISA @EXPORT);

#########################################################################
# Local customization

$community   = "public";         # Default SNMP community
$defbits     = 32;               # Default OID bits: 32/64
$output      = "targets.cfg";    # Output target file name
$router_file = "routers";        # Input list of devices to poll
$INFO        = 1;                # Print informational messages
$DEBUG       = 1;                # Print debug messages
$DEBUGCLASS  = 0;		# debug module class information
$DBOFF       = 0;                # Turn database queries off (debug)
$MODULEDIR   = "rtgmodules";
$MODULEORDERFILE = "$MODULEDIR/order.cfg";

@configs = ("rtg.conf", "/usr/local/rtg/etc/rtg.conf", "/etc/rtg.conf","/opt/rtg/etc/rtg.conf");

# No edits needed beyond this point
#########################################################################


# This Perl script requires the included SNMP modules
use lib ".:/opt/rtg/etc";
use BER;
use SNMP_Session;
use SNMP_util;

# This Perl script requires the not-included DBI module
use DBI;

use vars qw(%snmp_modules %table_map %table_name @statclasses %statbit %statvalue $currentclass);


# This Perl script requires the not-included DBI module
use DBI;

#
# Load all the modules we find
#
opendir(DIR,$MODULEDIR);
@modfiles = grep { /.*\.pl$/ } readdir(DIR);
closedir(DIR);

# Load all the added modules
foreach $mod_file (@modfiles) {
	debug("Now loading module $mod_file...");
	require "$MODULEDIR/$mod_file";
};


# List of "reserved" interfaces, i.e. those we don't care to monitor
# This list is inclusive of only Cisco/Juniper/Extreme
@reserved = (
    "tap",  "pimd", "pime", "ipip",
    "lo0",  "gre",  "pd-",  "pe-",  "gr-", "ip-",
    "vt-",  "mt-",  "mtun", "Null", "Loopback", "aal5",
    "-atm", "sc0", "T1", "Voice", 
    # these are for Extreme. The VLAN 000 is because no stats show up in
    # their VLAN interfaces, and they are all 000x for numbering
    "802.1Q", "rif", "VLAN 000"
);

foreach $conf (@configs) {
  if (open CONF, "<$conf") {
    print "Reading [$conf].\n" if $DEBUG;
    while ($line = <CONF>) {
      @cVals = split /\s+/, $line;
      if ($cVals[0] =~ /DB_Host/) {
        $db_host=$cVals[1];
      } elsif ($cVals[0] =~ /DB_User/) {
        $db_user=$cVals[1];
      } elsif ($cVals[0] =~ /DB_Pass/) {
        $db_pass=$cVals[1];
      } elsif ($cVals[0] =~ /DB_Database/) {
        $db_db=$cVals[1];
      } elsif ($cVals[0] =~ /Interval/) {
        $interval=$cVals[1];
      }
    }
    last;
  }
}


sub find_interface_id($$$$);
sub debug($);
sub print_target($$$$$$$$);
sub process_classes(@);
sub has_class($);
sub add_class($);
sub hasoid($$$);
sub getrouterbits($);

#
# Makes things easier with debugging from
# modules, even tho it's not as fast as doing
# the if $DEBUG before calling debug()
#
sub debug($) {
	$debug_line = shift;

	return 1 if($debug_line =~ /^Class:/ & $DEBUGCLASS == 0);

	print "$debug_line\n" if $DEBUG;
}

#
# get counter bits from config file to module that
# needs it
# 
sub getrouterbits($) {
	my $routername = shift;
	return $counterBits{$routername};

}
#
# If people use this instead of doing it themselves in their
# code, it makes it easier to change later on
#
sub print_target($$$$$$$$) {
	my ($router,$oid,$bits,$community,$table,$iid,$speed,$descr) = @_;

	($a,$a,$a,$a,@addrs) = gethostbyname($router);
	printf CFG "%d.%d.%d.%d\t", unpack('C4',$addrs[0]);
	print CFG "$oid\t";
	print CFG "$bits\t";
	print CFG "$community\t";
	print CFG "$table\t";
	print CFG "$iid\t";
	if($speed ne "") {
		print CFG "$speed\t";
	}
	print CFG "$descr\n";

}

# DBI SQL Insert Subroutine
sub sql_insert {
    ($sql) = @_;
    debug("SQL-: $sql");
    my $sth = $dbh->prepare($sql)
      or die "Can't prepare $sql: $dbh->errstr\n";
    my $rv = $sth->execute
      or die "can't execute the query: $sth->errstr\n";
}

# Find an RTG router id (rid) in the MySQL database.  If it doesn't
# exist, create a new entry and corresponding tables.
sub find_router_id {
    ($router) = @_;
    $sql = "SELECT DISTINCT rid FROM router WHERE name=\"$router\"";
    debug("SQL: $sql");
    my $sth = $dbh->prepare($sql)
      or die "Can't prepare $sql: $dbh->errstr\n";
    my $rv = $sth->execute
      or die "can't execute the query: $sth->errstr\n";
    if ( $sth->rows == 0 ) {
        print "No router id found for $router...";
        $sql = "INSERT INTO router (name) VALUES(\"$router\")";
        print "adding.\n";
        &sql_insert($sql);
        $rid = &find_router_id($router);
    }
    else {
        @row = $sth->fetchrow_array();
        $rid = $row[0];
    }
    $sth->finish;
    return $rid;
}

# Find an RTG interface id (iid) in the MySQL database.  If it doesn't
# exist, create a new entry.
sub find_interface_id($$$$) {
    ( $rid, $int, $desc, $speed ) = @_;
    $desc =~ s/ +$//g;    #remove trailing whitespace
    $sql = "SELECT id, description,speed FROM interface WHERE rid=$rid AND name=\"$int\"";
    debug("SQL: $sql"); 
    my $sth = $dbh->prepare($sql)
      or die "Can't prepare $sql: $dbh->errstr\n";
    my $rv = $sth->execute
      or die "can't execute the query: $sth->errstr\n";
    if ( $sth->rows == 0 ) {
        print "No id found for $int on device $rid...";
        $desc =~ s/\"/\\\"/g;    # Fix " in desc
        $sql = "INSERT INTO interface (name, rid, speed, description) VALUES(\"$int\", $rid, $speed, \"$desc\")";
        print "adding.\n";
        &sql_insert($sql);
        $iid = &find_interface_id( $rid, $int, $desc, $speed );
    }
    else {
        @row = $sth->fetchrow_array();
        $iid = $row[0];
        if ( $row[1] ne $desc ) {
            print "Interface description changed.\n";
            print "Was: \"$row[1]\"\n";
            print "Now: \"$desc\"\n";
            print "Suggest: UPDATE interface SET description='$desc' WHERE id=$iid\n";
        }
	if($row[2] ne $speed) {
            print "Interface speed changed.\n";
            print "Was: \"$row[2]\"\n";
            print "Now: \"$speed\"\n";
            print "Suggest: UPDATE interface SET speed='$speed' WHERE id=$iid\n";
	}
    }
    $sth->finish;
    return $iid;
}

sub process_classes(@) {
	@classes = @_;

	# remove any duplicates
	map { $classlist{$_} = 1; } @classes;

	$bit=0; $value=1;
	foreach(sort keys %classlist) {
		$statbit{$_} = $bit;
		$statvalue{$_} = $value;
		debug("Added stat $_ with bit $bit and value $value");
		$value *=2;
		$bit++;
	}

#	$currentclass = $statvalue{'disk'} + $statvalue{'cpu'} + $statvalue{'network'};
#	has_class("network");
}

sub has_class($) {
	my $classname = shift;

	my $foundclass = 0;

	#
	# we create a vector out of the current class value
	# and then turn it into a binary array, reversing it so
	# that bit order is the same order as in the original 
	# array. We then check to see if the bit location of the
	# requested class is set to 1, if so, we have already
	# added information for the requested class
	#
	vec($classvector,0,32) = $currentclass;
	my @bitlist = reverse split(//,unpack("B*",$classvector));
	if($bitlist[$statbit{$classname}] == 1) {
		debug("Class: $classname in $currentclass");
		$foundclass = 1;
	}
	else {	
		debug("Class: $classname not in $currentclass");
	}

	return $foundclass;
}

sub add_class($) {
	my $classname = shift;

	unless(has_class($classname)) {
		debug("Class: adding $classname");
		$currentclass += $statvalue{$classname};
	}
}

sub hasoid($$$) {
	my $comm = shift;
	my $router = shift;
	my $mib = shift;

	# we don't want messages about not finding the oid, it might
	# confuse/concern people as alot of these will fail
	#
	$SNMP_Session::suppress_warnings=2;

	my @result = snmpget ("$comm\@$router", $mib);

	# turn warnings back on
	$SNMP_Session::suppress_warnings=0;

	if($result[0] eq "") {
		debug("SNMP: $sstat not on $router...");
		return 0;
	}
	else { return 1; }

}

sub getmoduleorder($%) {
	my $systemtype = shift;
	my %module_list = @_;;

	my %current_list;
	my @final_order;

	my $count = 0;
	foreach $module (sort keys %module_list) {
		$current_list{$module} = $count;
		$count += 10;
	}

	foreach $key (keys %order_mod) { 
		if($systemtype =~ /$order_regex{$key}/) {
			debug("Changing order of $order_mod{$key} by $order_change{$key}...");
			$current_list{$order_mod{$key}} += $order_change{$key};
		}
	}

	foreach $module (sort { $current_list{$a} <=> $current_list{$b} } keys %current_list) {
			push(@final_order,$module);
	}

	return @final_order;

}


sub main {
    #
    # initial the classes vector information
    #
    process_classes(@statclasses);


    open ROUTERS, "<$router_file" or die "Could not open file: $router_file";
    while (<ROUTERS>) {
        chomp;
        s/ +$//g;    #remove space at the end of the line
        next if /^ *\#/;    #ignore comment lines
        next if /^ *$/;     #ignore empty lines
        if ( $_ =~ /(.+):(.+):(.+)/ ) {
            $r = $1;
            $c = $2;
            $b = $3;
            $communities{$r} = $c;
            $counterBits{$r} = $b;
            push @routers, $r;
        } elsif ( $_ =~ /(.+):(.+)/ ) {
            $r = $1;
            $c = $2;
            $communities{$r} = $c;
            push @routers, $r;
        } else {
            $communities{$_} = $community;
            push @routers, $_;
        }
    }
    close ROUTERS;

    open(MODULES, "$MODULEORDERFILE") || die "Can't open file $MODULEORDERFILE: $!\n"; 
    while(<MODULES>) {
	chomp;
	s/ +$//g;    #remove space at the end of the line
	next if /^ *\#/;    #ignore comment lines
	next if /^ *$/;     #ignore empty lines

	my($mod,$regex,$poschg) = /(.+):(.+):([+-]\d+)/;

	if($poschg eq "") {
		debug("ERROR: Ignoring order line $_...");

	} else {
		$order_mod{"$mod--$regex"} = $mod;
		$order_regex{"$mod--$regex"} = $regex;
		$order_change{"$mod--$regex"} = $poschg;
	}
    }

    close(MODULES);

    if ( $routers[0] eq "rtr-1.my.net" ) {
        print "\n** Error, $0 is not yet configured\n\n";
        print "Please edit the \"$router_file\" file and add network devices\n";
        exit(-1);
    }

    # SQL Database Handle
    if ( !$DBOFF ) {
       $dbh = DBI->connect("DBI:mysql:$db_db:host=$db_host",$db_user,$db_pass);
       if (!$dbh) {
          print "Could not connect to database ($db_db) on $db_host.\n";
          print "Check configuration.\n";
          exit(-1);
       }
    }

    open CFG, ">$output" or die "Could not open file: $!";
    ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) =
      localtime( time() );
    printf CFG "# Generated %02d/%02d/%02d %02d:%02d by $0\n", $mon + 1, $mday,
      $year + 1900, $hour, $min;
    print CFG "# Host\tOID\tBits\tCommunity\tTable\tID\tDescription\n";

    foreach $router (@routers) {
        $bits = getrouterbits{$router};
        # Sanity check bits
        $bits = $defbits if ( ( $bits != 32 ) && ( $bits != 64 ) );

        print "Poking $router ($communities{$router}) ($bits bit)...\n" if $INFO;
        @result = snmpget( "$communities{$router}\@$router", 'sysDescr' );
        $system = join ( ' ', @result );
        debug("System: $system");

	$rid = find_router_id($router);
	$session = SNMP_Session->open( $router, $communities{$router}, 161 )
          || die "Error opening SNMP session to $router";

	debug("Checking for modules information...");

	@module_ordering = getmoduleorder($system,%snmp_modules);
	
	$currentclass = 0;
	#foreach $mod_name (sort keys %snmp_modules) {
	foreach $mod_name (@module_ordering) {
		# we will have failures, suppress warnings to make output cleaner
		debug("Checking for $mod_name support");

		# we don't want messages about not finding the oid, it might
		# confuse/concern people as alot of these will fail
		#
		$SNMP_Session::suppress_warnings=2;

	 	@result = snmpget ("$communities{$router}\@$router", $snmp_modules{$mod_name});

		# turn warnings back on
		$SNMP_Session::suppress_warnings=0;

		if($result[0] ne "") {
			debug("Class: $currentclass");
			$module_func = "process_module_$mod_name";
			&$module_func($router,$communities{$router},$session);
		}
		debug("Class: $currentclass");

	}
    }
    close CFG;
    if ( !$DBOFF ) {
        $dbh->disconnect;
    }
    print "Done.\n";
}

main;
exit(0);
